(* portlandappdeleg.ml     Application delegate for Portland example
 *
 * Copyright (c) 2011 Psellos   http://psellos.com
 * Licensed under the MIT license:
 *     http://www.opensource.org/licenses/mit-license.php
 *
 * In this simple example, this class does all the work.
 *)

class t =
object (self)
    inherit Wrappee.t

    val mutable theLabel: UiLabel.t option = None
    val mutable theTimer: NsTimer.t option = None
    val mutable active: bool = true

    method label =
        match theLabel with
        | Some label -> label
        | None -> raise Not_found

    method setLabel' aLabel =
        theLabel <- Some aLabel

    method applicationDidFinishLaunching' (appl: UiApplication.t) =
        begin
        let timer =
            NsTimer.scheduledTimer5 0.25 self "timerTick:" Wrappee.nil true
        in
            theTimer <- Some timer;
        let dev = UiDevice.currentDevice ()
        in
            dev#beginGeneratingDeviceOrientationNotifications;
        self#display;
        end

    method applicationWillResignActive' (appl: UiApplication.t) =
        active <- false

    method applicationDidBecomeActive' (appl: UiApplication.t) =
        active <- true

    method private display =
        if active then
            let (angle, width, height, name) =
                match (UiDevice.currentDevice())#orientation with
                | UiDevice.OrientationUnknown ->
                    (0.0, 320.0, 460.0, "Unknown")
                | UiDevice.OrientationPortrait ->
                    (0.0, 320.0, 460.0, "Portrait")
                | UiDevice.OrientationPortraitUpsideDown ->
                    (3.1415926535897932, 320.0, 460.0, "Portrait Upside Down")
                | UiDevice.OrientationLandscapeLeft ->
                    (1.5707963267948966, 460.0, 320.0, "Landscape Left")
                | UiDevice.OrientationLandscapeRight ->
                    (-1.5707963267948966, 460.0, 320.0, "Landscape Right")
                | UiDevice.OrientationFaceUp ->
                    (0.0, 320.0, 460.0, "Face Up")
                | UiDevice.OrientationFaceDown ->
                    (0.0, 320.0, 460.0, "Face Down")
            in
                match theLabel with
                | None -> ()
                | Some label ->
                    begin
                    label#setText' (Printf.sprintf "\xce\xa8\n%s" name);
                    label#setTransform' (CgAffineTransform.makeRotation angle);
                    label#setBounds' (0.0, 0.0, width, height);
                    end

    method timerTick' (timer: NsTimer.t)  =
        self#display
end

let _ =
    let wrapped robjcv = let t = new t in let () = t#setContainer robjcv in t
    in
        Callback.register "Portlandappdeleg.wrapped" wrapped
